'Demo code#2 for DVI and VGA graphics shields
'Copyright (C) 2012 Rayslogic.com, LLC
'MIT License
'Demonstrates using custom fonts, graphics primitives, and Windows BMP images
'Requires SD card to show the Windows bitmap files
'If colors look wrong, be sure to check the board settings in the lcd driver 

CON 'Crystal settings
  _clkmode = xtal1+PLL16X
  _xinfreq = 5_000_000

OBJ
  lcd: "SSD1963_driver2b"   'NOTE:  Be sure to change pin settings in this driver to match your setup if not using Propeller Platform!
  sd: "fsrw"     'SD card driver

CON 'SD card pin settings      Make sure matches your setup!
  'Propeller Platform Express settings:
  SD_Do=0
  SD_Clk=1
  SD_Di=2
  SD_Cs=3
  {'PropBox Prop Settings
  SD_Do=4
  SD_Clk=5       
  SD_Di=7
  SD_Cs=6
  }
  
VAR 'buffer for bitmap line of pixels
  long lineBuffer[864*3/4]' buffer for line of an image  'needs to be long aligned
  long BmpPalette[256]

PUB Main |i

  'start VGA driver
  lcd.start

  'run demos
  repeat
    Text_Demo     'Shows how to easily draw text
    Scroll_Demo   'Shows how screen can scroll quickly using scroll function of SSD1963
    Font_Demo     'Shows how you can use custom fonts
    Demo2         'Shows how to use graphics primitives and also show bitmaps from SD card

PUB Text_Demo|i,j,k 'copies functionality of VGA_Text_Demo.Spin
  'Slightly modified to account for different font being used
  'Note:  This is not VGA Text Demo, is not 2.6kB, and is not using Internal ROM font!
  lcd.StartText   'Driver includes an 8x12 font from VGA_HIRES_TEXT (but only characters 0..127)
  lcd.out(0) 'clear screen
  
  lcd.str(string(13,"   VGA Text Demo...",13,13,$C,5," OBJ and VAR require only 2.6KB ",$C,1))
  repeat 14
    lcd.out(" ")
  repeat 4
    repeat i from $0E to 127
      lcd.out(i)
  lcd.str(string($C,6,"     Uses internal ROM font     ",$C,2))
  repeat 3000 'slight mod
    lcd.str(string($A,12,$B,14))
    lcd.hex(i++, 8)

PUB Scroll_Demo|i,j 'Shows how text scrolls on newline
  lcd.StartText 
  repeat 300
    repeat ((||?j)//80)
      i:= ((||?j)//100)+$E
      lcd.out(i)
    lcd.out(13)
    waitcnt(cnt+clkfreq/100)
  lcd.out(0) 'clear screen  
  lcd.ResetScroll 'reset scrolled screen to no scroll

PUB Font_Demo|i, j, k, c    

 'Change font
  lcd.SetFont(@Font1)   'Using an 8x16 font called "Progo"
 'Fill screen fast way
  lcd.FillScreen(lcd.RGB(1,64,128))

 'Draw characters all over the screen (80x30 characters) with random colors
 
  k:=0
  repeat 10
    repeat j from 0 to 29
      repeat i from 0 to 79
        lcd.SetFontColors(lcd.RGB(?c,?c,?c),lcd.RGB(?c,?c,?c)) 
        lcd.DrawChar(i*8,j*16,?c)
        k++
        k&=$FF


    
    
PUB Demo2|i,j,k
  
  'Fill screen fast way
  lcd.FillScreen(lcd.RGB(1,64,128))
  'lcd.DrawRectangle(lcd#rgb_white,0,0,639,479)     'draws white boarder around whole screen
   
  'Set random pixels
  repeat 10000
    lcd.SetPixel(lcd.RGB(?k,?k,?k),||?k//640,||?k//480)
   
  'Draw random lines
  repeat 500  
    lcd.DrawLine(lcd.RGB(?k,?k,?k),||?k//640,||?k//480,||?k//640,||?k//480)
    
  'Draw random circles
  repeat 100
    lcd.DrawSolidCircle(lcd.RGB(?k,?k,?k),40+||?k//560,40+||?k//400,||?k//40)
   
  'Draw random triangles
  repeat 50
    lcd.DrawSolidTriangle(lcd.RGB(?k,?k,?k),||?k//640,||?k//480,||?k//640,||?k//480,||?k//640,||?k//480)
   
  'Draw random rectangles
  repeat 50
    lcd.DrawSolidRectangle(lcd.RGB(?k,?k,?k),||?k//640,||?k//480,||?k//640,||?k//480)
   
  lcd.StartText   'Driver includes an 8x12 font from VGA_HIRES_TEXT (but only characters 0..127)
  lcd.out(0) 'clear screen
  lcd.str(string("Attempting to mount SD card to show BMP images...",13))
   
  'Mounting SD card now
  repeat   'try a few times to mount the SD card
    i:=sd.mount_explicit(SD_DO, SD_CLK, SD_DI,SD_CS)
    if i==0
       quit
    waitcnt(cnt+clkfreq/4)
    lcd.out(".")
   
  'Showing some 24-bit bitmap images from SD card
  k:=string("test1.bmp")   
  repeat j from "1" to "4"
    byte[k+4]:=j   'alter bitmap filename to pick different bitmaps
    BMP(0,0,k)   'show the bitmap
    waitcnt(cnt+clkfreq*2)
 
      
  
    

PUB BMP(bmpX,bmpY,sFilename)|i,j,k,l
'Draw a bitmap file onto the screen
    'Update speed seems to be only limited by SD card reading speed
    'note that bitmaps are stored from bottom line to top line!
    i:=\sd.popen(sFilename,"r") 'open image for reading
    if i<>0
      return i   
    sd.pread(@bfType,2) 'read bmp header
    sd.pread(@bfSize,4) 'read bmp header  
    sd.pread(@bfReserved1,4) 'read bmp header  
    sd.pread(@bfOffBits,16) 'read bmp header
    sd.pread(@biPlanes,4)
    sd.pread(@biCompression,24)

    if (biHeight<1) or (biWidth<1)
      return -1  'some kind of problem

    l:=biHeight+bmpY-1  
      
    case biBitCount
      24:  'true color (3 bytes per pixel)        
        i:=(biWidth*3)//4
        k:=biWidth*3+i     
        repeat j from l to bmpY 
          'process image row by row          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,24,0)
 
      8:  '256-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54
        'read in palette
        sd.pread(@BmpPalette,i)
        i:=biWidth//4
        k:=biWidth+i
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,8,@BmpPalette) 
      
      4:  '16-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54        
        'read in palette
        sd.pread(@BmpPalette,i)
        'now paint picture
        i:=(biWidth>>1)//4
        k:=(biWidth>>1)+i 
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,4,@BmpPalette) 

    sd.pclose
    return 0



DAT     'Data for reading bitmap files

BMPHeader  byte 'Mostly using info from here:  http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
bfType byte "B","M"  ' 19778
bfSize long 0
bfReserved1 word 0
bfReserved2 word 0
bfOffBits long 54
biSize long 40
biWidth long 0
biHeight long 10
biPlanes word 1
biBitCount word 24
biCompression long 0
biSizeImage long 0
biXPelsPerMeter long 0
biYPelsPerMeter long 0
biClrUsed long 0
biClrImportant long 0

DAT  'Font:  Custom font located here.  Other fonts can be selected with the SetFont command
'Note:  If you do not need a font, you can save space by commenting out the file command line below
font1 byte
file "ProgoCo_wHeader.dat"   'nice 8x16 fixed pitch font
